// TypeScript (ResultGrid.tsx snippet)
import React, { useEffect, useState } from 'react';
import { DataGrid, GridColDef } from '@mui/x-data-grid';
import { OpenAPI, SamplesService } from '../../openapi';
import './SampleImage.css';


interface ImageInfo {
    id: number;
    filepath: string;
    comment?: string;
}

interface SampleResult {
    sample_id: number;
    sample_name: string;
    puck_name?: string;
    dewar_name?: string;
    images: ImageInfo[];
}

interface ResultGridProps {
    activePgroup: string;
}

const ResultGrid: React.FC<ResultGridProps> = ({ activePgroup }) => {
    const [rows, setRows] = useState<SampleResult[]>([]);
    const [basePath, setBasePath] = useState('');

    useEffect(() => {
        const mode = import.meta.env.MODE;
        OpenAPI.BASE =
            mode === 'test'
                ? import.meta.env.VITE_OPENAPI_BASE_TEST
                : mode === 'prod'
                    ? import.meta.env.VITE_OPENAPI_BASE_PROD
                    : import.meta.env.VITE_OPENAPI_BASE_DEV;

        if (!OpenAPI.BASE) {
            console.error('OpenAPI.BASE is not set. Falling back to a default value.');
            OpenAPI.BASE = 'https://default-url.com';
        }

        console.log('Environment Mode:', mode);
        console.log('Resolved OpenAPI.BASE:', OpenAPI.BASE);

        // Update the base path for images
        setBasePath(`${OpenAPI.BASE}/`);
    }, []);

    useEffect(() => {
        const fetchData = () => {
            console.log("Fetching sample results for active_pgroup:", activePgroup);
            SamplesService.getSampleResultsSamplesResultsGet(activePgroup)
                .then((response: SampleResult[]) => {
                    console.log("Response received:", response);
                    setRows(response);
                })
                .catch((err: Error) => {
                    console.error("Error fetching sample results:", err);
                });
        };

        // Fetch data initially.
        fetchData();

        // Set up an interval to refresh data every 15 seconds (adjust as needed)
        const intervalId = setInterval(fetchData, 15000);

        // Clean up when component unmounts or activePgroup changes.
        return () => clearInterval(intervalId);
    }, [activePgroup]);


    const columns: GridColDef[] = [
        { field: 'sample_id', headerName: 'ID', width: 70 },
        { field: 'sample_name', headerName: 'Sample Name', width: 150 },
        { field: 'puck_name', headerName: 'Puck Name', width: 150 },
        { field: 'dewar_name', headerName: 'Dewar Name', width: 150 },
        {
            field: 'images',
            headerName: 'Images',
            width: 300,
            renderCell: (params) => {
                const imageList: ImageInfo[] = params.value;
                if (!imageList || imageList.length === 0) {
                    return null;
                }

                // Filter the images to include only the two bb_raster images
                const filteredImages = imageList.filter(
                    (img) =>
                        img.filepath.includes("bb_raster_0") ||
                        img.filepath.includes("bb_raster_90")
                );

                if (filteredImages.length === 0) {
                    return null;
                }

                return (
                    <div style={{ display: 'flex', alignItems: 'center' }}>
                        {filteredImages.map((img) => {
                            const url = basePath + img.filepath;
                            return (
                                <img
                                    key={img.id}
                                    src={url}
                                    alt={img.comment || 'sample'}
                                    className="zoom-image"
                                    style={{
                                        width: 40,
                                        height: 40,
                                        marginRight: 5,
                                        borderRadius: 4,
                                    }}
                                />
                            );
                        })}
                    </div>
                );
            },
        },

    ];

    return (
        <DataGrid
            rows={rows}
            columns={columns}
            pageSize={10}
            getRowId={(row) => row.sample_id}
            sx={{
                '& .MuiDataGrid-cell': {
                    overflow: 'visible',
                },
            }}
        />

    );
};

export default ResultGrid;